@@ -1,7 +1,7 @@
-/* 
- * Copyright (C) 2007-2010 B.A.T.M.A.N. contributors:
+/*
+ * Copyright (C) 2007-2011 B.A.T.M.A.N. contributors:
  *
- * Andreas Langer <a.langer@q-dsl.de>, Marek Lindner <lindner_marek@yahoo.de>
+ * Andreas Langer <an.langer@gmx.de>, Marek Lindner <lindner_marek@yahoo.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -34,9 +34,11 @@
 #include "functions.h"
 #include "packet.h"
 #include "bat-hosts.h"
+#include "debugfs.h"
 
 
 #define TTL_MAX 50
+#define NUM_PACKETS 3
 
 
 void traceroute_usage(void)
@@ -47,7 +49,7 @@ void traceroute_usage(void)
 	printf(" \t -n don't convert addresses to bat-host names\n");
 }
 
-int traceroute(int argc, char **argv)
+int traceroute(char *mesh_iface, int argc, char **argv)
 {
 	struct icmp_packet icmp_packet_out, icmp_packet_in;
 	struct bat_host *bat_host;
@@ -58,7 +60,9 @@ int traceroute(int argc, char **argv)
 	char *dst_string, *mac_string, *return_mac, dst_reached = 0;
 	int ret = EXIT_FAILURE, res, trace_fd = 0, i;
 	int found_args = 1, optchar, seq_counter = 0, read_opt = USE_BAT_HOSTS;
-	double time_delta;
+	double time_delta[NUM_PACKETS];
+	char *debugfs_mnt;
+	char icmp_socket[MAX_PATH+1];
 
 	while ((optchar = getopt(argc, argv, "hn")) != -1) {
 		switch (optchar) {
@@ -82,7 +86,7 @@ int traceroute(int argc, char **argv)
 	}
 
 	dst_string = argv[found_args];
-	bat_hosts_init();
+	bat_hosts_init(read_opt);
 	bat_host = bat_hosts_find_by_name(dst_string);
 
 	if (bat_host)
@@ -99,11 +103,19 @@ int traceroute(int argc, char **argv)
 
 	mac_string = ether_ntoa_long(dst_mac);
 
-	trace_fd = open(BAT_DEVICE, O_RDWR);
+	debugfs_mnt = debugfs_mount(NULL);
+	if (!debugfs_mnt) {
+		printf("Error - can't mount or find debugfs\n");
+		goto out;
+	}
+
+	debugfs_make_path(SOCKET_PATH_FMT, mesh_iface, icmp_socket, sizeof(icmp_socket));
+
+	trace_fd = open(icmp_socket, O_RDWR);
 
 	if (trace_fd < 0) {
-		printf("Error - can't open a connection to the batman adv kernel module via the device '%s': %s\n",
-				BAT_DEVICE, strerror(errno));
+		printf("Error - can't open a connection to the batman adv kernel module via the socket '%s': %s\n",
+				icmp_socket, strerror(errno));
 		printf("Check whether the module is loaded and active.\n");
 		goto out;
 	}
@@ -118,12 +130,15 @@ int traceroute(int argc, char **argv)
 		dst_string, mac_string, TTL_MAX, sizeof(icmp_packet_out));
 
 	for (icmp_packet_out.ttl = 1; !dst_reached && icmp_packet_out.ttl < TTL_MAX; icmp_packet_out.ttl++) {
+		return_mac = NULL;
+		bat_host = NULL;
 
-		for (i = 0; i < 3; i++) {
+		for (i = 0; i < NUM_PACKETS; i++) {
 			icmp_packet_out.seqno = htons(++seq_counter);
+			time_delta[i] = 0.0;
 
 			if (write(trace_fd, (char *)&icmp_packet_out, sizeof(icmp_packet_out)) < 0) {
-				printf("Error - can't write to batman adv kernel file '%s': %s\n", BAT_DEVICE, strerror(errno));
+				printf("Error - can't write to batman adv kernel file '%s': %s\n", icmp_socket, strerror(errno));
 				continue;
 			}
 
@@ -137,19 +152,13 @@ int traceroute(int argc, char **argv)
 
 			res = select(trace_fd + 1, &read_socket, NULL, NULL, &tv);
 
-			if (res <= 0) {
-				if (i == 0)
-					printf("%2hu: ", icmp_packet_out.ttl);
-
-				printf(" * ");
-				fflush(stdout);
+			if (res <= 0)
 				continue;
-			}
 
 			read_len = read(trace_fd, (char *)&icmp_packet_in, sizeof(icmp_packet_in));
 
 			if (read_len < 0) {
-				printf("Error - can't read from batman adv kernel file '%s': %s\n", BAT_DEVICE, strerror(errno));
+				printf("Error - can't read from batman adv kernel file '%s': %s\n", icmp_socket, strerror(errno));
 				continue;
 			}
 
@@ -161,34 +170,22 @@ int traceroute(int argc, char **argv)
 
 			switch (icmp_packet_in.msg_type) {
 			case ECHO_REPLY:
+				dst_reached = 1;
+				/* fall through */
 			case TTL_EXCEEDED:
-				time_delta = end_timer();
+				time_delta[i] = end_timer();
 
-				if (i > 0) {
-					printf("  %.3f ms", time_delta);
-					break;
+				if (!return_mac) {
+					return_mac = ether_ntoa_long((struct ether_addr *)&icmp_packet_in.orig);
+
+					if (read_opt & USE_BAT_HOSTS)
+						bat_host = bat_hosts_find_by_mac((char *)&icmp_packet_in.orig);
 				}
 
-				return_mac = ether_ntoa_long((struct ether_addr *)&icmp_packet_in.orig);
-
-				bat_host = NULL;
-				if (read_opt & USE_BAT_HOSTS)
-					bat_host = bat_hosts_find_by_mac((char *)&icmp_packet_in.orig);
-
-				if (!bat_host)
-					printf("%2hu: %s %.3f ms",
-						icmp_packet_out.ttl, return_mac, time_delta);
-				else
-					printf("%2hu: %s (%s) %.3f ms",
-						icmp_packet_out.ttl, bat_host->name, return_mac, time_delta);
-
-				if (icmp_packet_in.msg_type == ECHO_REPLY)
-					dst_reached = 1;
-
 				break;
 			case DESTINATION_UNREACHABLE:
 				printf("%s: Destination Host Unreachable\n", dst_string);
-				break;
+				goto out;
 			case PARAMETER_PROBLEM:
 				printf("Error - the batman adv kernel module version (%d) differs from ours (%d)\n",
 						icmp_packet_in.ttl, COMPAT_VERSION);
@@ -200,6 +197,18 @@ int traceroute(int argc, char **argv)
 			}
 		}
 
+		if (!bat_host)
+			printf("%2hhu: %s", icmp_packet_out.ttl, (return_mac ? return_mac : "*"));
+		else
+			printf("%2hhu: %s (%s)", icmp_packet_out.ttl, bat_host->name, return_mac);
+
+		for (i = 0; i < NUM_PACKETS; i++) {
+			if (time_delta[i])
+				printf("  %.3f ms", time_delta[i]);
+			else
+				printf("   *");
+		}
+
 		printf("\n");
 	}
 
